#include <iostream>
#include <stdlib.h>
#include <math.h>

#include "allegro5/allegro.h"
#include "allegro5/allegro_image.h"
#include "allegro5/allegro_primitives.h"
#include "allegro5/allegro_font.h"

#include "MyWorld.h"
#include "PlatformerWorld.h"
#include "Rectangle.h"


using namespace std;

/*this boolean is true when the program has stopped*/
bool programFinished;

/*Objects of the library Allegro (queue of message such keyboards etc., the timer and the display)*/
ALLEGRO_EVENT_QUEUE* event_queue;
ALLEGRO_TIMER* timer;
ALLEGRO_DISPLAY* display;


/*enumeration of interesting keys of the keyboard*/
enum MYKEYS
{
    KEY_UP, KEY_DOWN, KEY_LEFT, KEY_RIGHT
};



/*this array states for keys in MYKEYS whether it is pressed or not*/
bool key[4] = { false, false, false, false };



/*the scene*/
PlatformerWorld* world;

/*print error message*/
void errorMessageShow(const char* message)
{
    cout << message;
    exit(1);
}








/*clean the memory*/
void shutdown(void)
{
    delete world;

    if (timer)
        al_destroy_timer(timer);

    if (display)
        al_destroy_display(display);

    if (event_queue)
        al_destroy_event_queue(event_queue);
}





/*initialize Allegro*/

void init(void)
{

    if (!al_init())
        errorMessageShow("Failed to initialize allegro");

    al_set_new_display_flags(ALLEGRO_WINDOWED);
    display = al_create_display(640, 480);
    if (!display)
        errorMessageShow("Failed to create display");


    if (!al_install_keyboard())
        errorMessageShow("Failed to install keyboard");

    timer = al_create_timer(1.0 / 60);
    if (!timer)
        errorMessageShow("Failed to create timer");


    event_queue = al_create_event_queue();
    if (!event_queue)
        errorMessageShow("Failed to create event queue");

    al_register_event_source(event_queue, al_get_keyboard_event_source());
    al_register_event_source(event_queue, al_get_timer_event_source(timer));
    al_register_event_source(event_queue, al_get_display_event_source(display));

    programFinished = false;

    world = new MyWorld();

}








/*connect keyboard to the world (the scene)*/
void user_input(ALLEGRO_EVENT event)
{
    if (key[KEY_LEFT])
    {
        world->playerMoveLeft();

    }

    if (key[KEY_RIGHT])
    {
        world->playerMoveRight();
    }

    if (key[KEY_UP])
    {
        world->playerJump();
    }
}


/*the loop of the game*/
void game_loop(void)
{
    bool redraw = true;
    al_start_timer(timer);

    while (!programFinished)
    {
        ALLEGRO_EVENT event;
        al_wait_for_event(event_queue, &event);

        if (event.type == ALLEGRO_EVENT_TIMER)
        {
            redraw = true;

        }

        else if (event.type == ALLEGRO_EVENT_DISPLAY_CLOSE)
        {
            programFinished = true;

        }
        else if(world->isGameover())
        {
            programFinished = true;
        }


        else if (event.type == ALLEGRO_EVENT_KEY_DOWN)
        {
            if (event.keyboard.keycode == ALLEGRO_KEY_ESCAPE)
            {
                programFinished = true;
            }
            else
            {

                switch(event.keyboard.keycode)
                {
                case ALLEGRO_KEY_UP:
                    key[KEY_UP] = true;
                    break;

                case ALLEGRO_KEY_DOWN:
                    key[KEY_DOWN] = true;
                    break;

                case ALLEGRO_KEY_LEFT:
                    key[KEY_LEFT] = true;
                    break;

                case ALLEGRO_KEY_RIGHT:
                    key[KEY_RIGHT] = true;
                    break;
                }

            }

        }
        else if (event.type == ALLEGRO_EVENT_KEY_UP)
        {
            switch(event.keyboard.keycode)
            {
            case ALLEGRO_KEY_UP:
                key[KEY_UP] = false;
                break;

            case ALLEGRO_KEY_DOWN:
                key[KEY_DOWN] = false;
                break;

            case ALLEGRO_KEY_LEFT:
                key[KEY_LEFT] = false;
                break;

            case ALLEGRO_KEY_RIGHT:
                key[KEY_RIGHT] = false;
                break;
            }




        }

        user_input(event);

        if (redraw && al_is_event_queue_empty(event_queue))
        {
            redraw = false;
            world->step();
            world->draw();

            al_flip_display();
        }
    }
}



/*the main function*/
int main(int argc, char* argv[])
{

    init();
    game_loop();
    shutdown();

}
